# Copyright (c) 2012 Santosh Philip

# This file is part of eppy.

# Eppy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Eppy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with eppy.  If not, see <http://www.gnu.org/licenses/>.

"""some init routines to do the documentation"""

from IPython.display import Image
from IPython.display import display_pretty, display_html, display_jpeg, display_png, display_json, display_latex, display_svg
idfeditor = Image(filename='../docs/images/idf3.png')
material_lists = Image(filename='../docs/images/material_lists.png')
html_snippet1 = Image(filename='../docs/images/html1.png')
html_snippet2 = Image(filename='../docs/images/html2.png')
plantloop1 = Image(filename='../docs/images/hhh1.png')
plantloop2 = Image(filename='../docs/images/hhh_new.png')
diffcsv = Image(filename='../docs/images/diffcsv.tiff')